/**\file trig.h
*\brief Real trigonometric functions. */

#ifndef TRIG_H_
#define TRIG_H_
#include "unifloat.h"

/**\brief The sine of the given Unifloat number.
*
*\return The sine of \b x.
*\n The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* sin_UF(Unifloat* x);

/**\brief The cosine of the given Unifloat number.
*
*\return The cosine of \b x.
*\n The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* cos_UF(Unifloat* x);

/**\brief The tangent of the given Unifloat number.
*
*\return The tan of \b x.
*\n The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* tan_UF(Unifloat* x);

/**\brief The arc sine of the given Unifloat number.
*
*\return The arc sine of \b x.
*\n The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* asin_UF(Unifloat* x);

/**\brief The arc cosine of the given Unifloat number.
*
*\return The arc cosine of \b x.
*\n The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* acos_UF(Unifloat* x);

/**\brief The arc tangent of the given Unifloat number.
*
*\return The arc tangent of \b x.
*\n The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* atan_UF(Unifloat* x);

/**\brief The arc tangent of Unifloat number divided by another one.
*
*\return The arc tangent of <b>x/y</b> .
*\n The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* atan2_UF(Unifloat* x, Unifloat* y);

/**\brief Make a number to be between 0 and 2*Pi.
*
*\return The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* arrangeArgument_UF(Unifloat* x);

/**\brief Auxiliary function needed to calculate atan iteratively.
*
*\return The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* calcAtan_UF(Unifloat* x, Unifloat* y, int i);

/**\brief Auxiliary function needed to calculate tan iteratively.
*
*\return The function creates and returns the object that have to be removed later using \b delete_UF. */
Unifloat* calcTan_UF(Unifloat* x, int i);
#endif //TRIG_H_
